variable "tags" {
  description = "Helpful tags for resource organization."
  type        = map(any)
  default     = {}
}

variable "region" {
  description = "Where you want to deploy your resources."
  type        = string
  default     = "us-east-1"
}

variable "disaster_zones" {
  description = "A list of availability zones for resource redundancy."
  type        = list(string)
  default     = []
}

variable "vpc_cidr" {
  description = "The VPC subnet that supports the entire application plane."
  type        = string
  default     = ""
}

variable "public_cidrs" {
  description = "This subnet will support VMs with a route to the public internet."
  type        = list(string)
  default     = []
}

variable "private_cidrs" {
  description = "This subnet will support VMs with DB or internal access."
  type        = list(string)
  default     = []
}

variable "exposed_ports" {
  description = "A list of ports that are punched through the firewall."
  type        = list(number)
  default     = []
}

variable "ssh_public_key" {
  description = "A public key signature for secure shell access."
  type        = string
  default     = ""
}

variable "ec2_template" {
  description = "A structured template for mostly uniform EC2s."
  type = list(object({
    hostname   = string
    ami        = string
    elastic_ip = bool
    family     = string
    disksize   = number
    subnet     = string
  }))
}

variable "db_template" {
  description = "A structured template for mostly uniform Amazon RDSes."
  type = list(object({
    name           = string
    engine         = string
    engine_version = number
    class          = string
    family         = string
    subnet         = string
  }))
}
